#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   mathHelper.py
@Time    :   2019/02/27
@Author  :   Yaron Huang 
@Version :   1.0
@Contact :   yaronhuang@qq.com
@Desc    :   Math Common Function
'''


